;  Motor - Controlling two Motors using an AT89C2051 and Serial Communications
;
;  This Application provides a simple interface to a PC's Terminal Emulator (and
;   Visual Basic Program) for Controlling a toy crane in real time.  Motors will
;   run with PWM signal running at a 50% duty cycle.  For the initial application
;   the motors will be powered by a 9 Volt radio battery.  The normal voltage
;   applied is three volts.  By halving the duty cycle, an average voltage of 4.5
;   volts will be applied.  This voltage will be dropped by 1.5 Volts using the
;   293D motor controller chip.  
;
;  Serial Commands:
;
;  #F - Run Motor # Forward.  CR/LF Returned.
;  #B - Run Motor # Back.  CR/LF Returned.  
;  #S - Stop the Operation of Motor #.  CR/LF Returned.
;
;  Timer 1 Interrupt Rate: 75.76 KHz (for a 18.94 KHz PWM Frequency) 
;
;  Serial Interface Speed: 2400 bps (Actually) 
;
;  Myke Predko
;  98.04.19
;
;  Hardware Notes:
;  AT89C2051 is used as the Microcontroller
;   - Oscillator Speed is 20 MHz
;  Serial Output is on P3.1
;  Serial Input is on P3.0
;  LED Control is at P1.5
;  Motor PWM is p1.3
;  Motor 1 + Control is P1.6
;  Motor 1 - Control is P1.2
;  Motor 2 + Control is P1.7
;  Motor 2 - Control is P1.4

;  Constant Declarations
TimerReload EQU 0EAh            ;  Value to Reload the Timer With

;  Variable Declarations
Counter     EQU 020h            ;  Real Time Counter
Motor       EQU 6               ;  Motor Specified
Direction   EQU 7               ;  Direction to Turn Wheels in


;  Macros
MACRO SendChar
@Loop                           ;  Loop Here Until Previous Sent
  jnb     TI,@Loop
  mov     SBUF,A                ;  Output the Character
  clr     TI                    ;  Clear TI for the Next TX
ENDMAC


;  Mainline
 org 0                          ;  Execution Starts Here

  ajmp    MainLine              ;  Jump Over to the Mainline

 org 01Bh                       ;  Timer 1 - PWM Handler

  push    PSW
 
  inc     Counter               ;  Increment the Counter

  mov     C, 1                  ;  Use Bit 1 of the Counter as
  mov     P1.3, C               ;   the PWM Output

  pop     PSW

  reti                          ;  Return from the Interrupt
 

Mainline:                       ;  Mainline Code, Wait for Data and Move
                                ;   the Motors Accordingly

  mov     P1,#%00100011         ;  Turn Off the Motors Initially
  
  mov     R0,#0

  mov     Counter,#0            ;  Clear all the Operation Variables Bytes
  mov     Motor,#0
  mov     Direction,#0

  mov     SCON,#%01010000       ;  Run Serial Port in 8 Bit Mode
  mov     TMOD,#%00100001       ;  Run the Timers, Tmr1 in 8 Bit Reload Mode
  mov     TH1,#TimerReload      ;  Enable the Timer
  mov     TL1,#TimerReload
  orl     TCON,#040h            ;  Enable Tmr1 to Run
  anl     PCON,#07Fh            ;  Make Sure SMOD is Reset

  mov     IE,#%10001000         ;  Enable the Timer 1 Interrupt

  mov     SBUF,#8               ;  Send Initial BackSpace for Setup

Loop:                           ;  Loop Here to Transmit Each Character
  jnb     RI,Loop               ;  Was Anything Received?  

  clr     RI                    ;  Yes, Clear RI
  mov     A,SBUF                ;  Save the Character Received

  cjne    A,#00Dh,NotCR         ;  Was CR Sent?

  cjne    R6,#'1',CR2Check      ;  Check the Motor Number
  ajmp    CR1DirCheck
CR2Check:
  cjne    R6,#'2',CRBad         ;  Not '1' or '2', Invalid
  ajmp    CR2DirCheck
  
CR1DirCheck:                    ;  Do the Direction Check for Motor 1
  mov     A,P1
  anl     A,#%10111011          ;  Clear the Motor Bits
  cjne    R7,#'F',CR1BCheck     ;  Check for Forward
  orl     A,#%01000000          ;  Go Forwards
  mov     P1,A
  ajmp    CREnd                 
CR1BCheck:
  cjne    R7,#'B',CR1SCheck     ;  Check for Back
  orl     A,#%00000100
  mov     P1,A
  ajmp    CREnd
CR1SCheck:
  cjne    R7,#'S',CRBad         ;  Check for Stop
  mov     P1,A
  ajmp    CREnd
  
CR2DirCheck:                    ;  Do the Direction Check for Motor 2
  mov     A,P1
  anl     A,#%01101111          ;  Clear the Motor Bits
  cjne    R7,#'F',CR2BCheck     ;  Check for Forward
  orl     A,#%10000000
  mov     P1,A
  ajmp    CREnd                 
CR2BCheck:
  cjne    R7,#'B',CR2SCheck     ;  Check for Back
  orl     A,#%00010000
  mov     P1,A
  ajmp    CREnd
CR2SCheck:
  cjne    R7,#'S',CRBad         ;  Check for Stop
  mov     P1,A
  ajmp    CREnd

CRBad:                          ;  Invalid Character Sent
  mov     A,#'-'                ;  Mark It
  SendChar
CREnd:                          ;  Finished with the Command
  mov     A,#00Dh               ;  Send the "CR" Back with a LF
  SendChar
  mov     A,#00Ah               
  SendChar
  mov     R6,#0                 ;  Clear the Motor/Direction Registers
  mov     R7,#0
  ajmp    Loop


NotCR:                          ;  Was LF Sent?
  cjne    A,#00Ah,NotLF
  SendChar                      ;  Just Reply
  ajmp    Loop                  


NotLF:                          ;  Was BS Sent?
  cjne    A,#008h,NotBS

  cjne    R6,#0,DoBSMot         ;  Something in Motor?
  cjne    R7,#0,DoBSDir         ;  Something in Direction?
  ajmp    Loop                  ;   Nothing, Just Return

DoBSMot:                        ;  Backspace at Motor    
  mov     Motor,#0
  SendChar
  ajmp    Loop

DoBSDir:                        ;  Backspace at Direction    
  mov     Direction,#0
  SendChar
  ajmp    Loop


NotBS:
  cjne    R6,#0,NotMotor        ;  Motor Specification?
  mov     Motor,A               ;  Save the Motor
  SendChar
  ajmp    Loop


NotMotor:                       ;  Check for the Direction
  cjne    R7,#0,NotDir          ;   - Not Direction, Loop Around
  mov     Direction,A           ;  Save the Direction
  SendChar
  ajmp    Loop                  ;  Wait to Transmit the Next Character

NotDir:                         ;  Not Anything Valid, Loop Around
  ajmp    Loop                  ;   - Short Jump is Out of Range
